package interrupt;

//run this a few times
public class ThreadInterruptExample {

    public static void main(String[] args) {
        // Create a new thread that simulates a long-running task
        Thread workerThread = new Thread(new Runnable() {
            @Override
            public void run() {
                try {
                    // Simulate a task that runs until the thread is interrupted
                    for (int i = 0; i < 5; i++) {
                        System.out.println("Working... iteration " + (i + 1));
                        
                        // Check for interruption before calling sleep
                        if (Thread.interrupted()) {
                            System.out.println("Thread was interrupted before sleep. Exiting...");
                            return; // Exit the loop and the thread if interrupted
                        }
                        
                        // Simulate work by sleeping for 1 second
                        Thread.sleep(1000); // This could throw InterruptedException
                    }
                } catch (InterruptedException e) {
                    // Handle the interruption gracefully during sleep
                    System.out.println("Thread was interrupted during sleep. Exiting...");
                    Thread.currentThread().interrupt(); // Preserve the interrupt flag
                } finally {
                    // Always execute cleanup code, regardless of interruption
                    System.out.println("Cleaning up resources...");
                }
            }
        });

        // Start the worker thread
        workerThread.start();

        try {
            // Let the worker thread run for 2 seconds before interrupting it
            Thread.sleep(2000);
            System.out.println("Main thread interrupting worker thread...");
            workerThread.interrupt(); // Interrupt the worker thread after 2 seconds
        } catch (InterruptedException e) {
            e.printStackTrace();
        }

        // Wait for the worker thread to finish
        try {
            workerThread.join(); // Wait for the worker thread to complete
        } catch (InterruptedException e) {
            e.printStackTrace();
        }

        System.out.println("Main thread completed.");
    }
}

