package join;

public class ThreadJoin extends Thread {

    public void run() {
        for (int i = 1; i <= 3; i++) {
            try {
                // Sleep for a random time up to 3 seconds
                sleep((int) (Math.random() * 3000));
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
            System.out.println(i);
        }
    }

    public static void main(String[] args) {
        Thread thread1 = new ThreadJoin();
        Thread thread2 = new ThreadJoin();

        thread1.start();  // Start thread1
        thread2.start();  // Start thread2

        try {
            // Wait for thread1 to finish
            thread1.join();

            // Wait for thread2 to finish
            thread2.join();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }

        System.out.println("Done");  // Only prints "Done" after both threads are completely finished

        /*
         * Explanation:
         * - After starting thread1 and thread2, main thread calls join() on each.
         * - join() causes the main thread to pause execution until that particular thread has finished.
         * - thread1.join() --> main thread waits until thread1 is completely done.
         * - thread2.join() --> after that, main thread waits until thread2 is completely done.
         * - Only *after* both thread1 and thread2 are finished, the main thread prints "Done".
         * - This guarantees that "Done" will be the last thing printed.
         */
    }
}

