package join;

//Run this program a few times to see variations of output 
public class ThreadNoJoin extends Thread {

    public void run() {
        for (int i = 1; i <= 3; i++) {
            try {
                // Sleep for a random time up to 3 seconds
                sleep((int) (Math.random() * 3000));
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
            System.out.println(i);
        }
    }

    public static void main(String[] args) {
        Thread thread1 = new ThreadNoJoin();
        Thread thread2 = new ThreadNoJoin();

        thread1.start();  // Start thread1 (it begins running independently)
        thread2.start();  // Start thread2 (also runs independently)

        // No join! So main does NOT wait for thread1 or thread2 to finish.
        System.out.println("Done");  // Main immediately prints "Done" and may exit before threads are finished

        /*
         * Explanation:
         * - After calling start(), the main thread does NOT block or wait for the other threads.
         * - The thread scheduler decides when thread1 and thread2 actually run.
         * - "Done" may be printed before any or all of the numbers (1, 2, 3) from the threads.
         * - Main thread can exit early, even though thread1 and thread2 are still running.
         * - To make main wait for the threads to finish, we would need to call thread1.join() and thread2.join().
         */
    }
}
