package messageThreadRunnable;

/**
 * This class implements Runnable to define a task that can be executed by a thread.
 * It prints a message multiple times.
 * This is the preferred way to define threads in Java.
 */
public class DisplayMessageTask implements Runnable {
    // The message to display and how many times to display it
    private String msg;
    private int times;

    /**
     * Constructor to initialize the message and number of repetitions.
     *
     * @param msg   the message to print
     * @param times how many times to print the message
     */
    public DisplayMessageTask(String msg, int times) {
        this.msg = msg;
        this.times = times;
    }

    /**
     * This method contains the code that will be run by the thread.
     * It simply prints the message multiple times.
     * Note: Threads using Runnable must be created separately with new Thread(...).
     */
    public void run() {
        for (int i = 0; i < times; i++) {
            System.out.println(msg);
        }
    }
}
